/*
Copyright (c) 2015  V.Ayllón.  All rights reserved.
 
 TwinTeethMC is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
 to anyone for the consequences of using it or for whether it serves any
 particular purpose or works at all, unless he says so in writing.  Refer
 to the GNU General Public License, Version 2 or later, for full details.
 
 Everyone is granted permission to copy, modify and redistribute
 TwinTeethMC, but only under the conditions described in the GNU General Public
 License.  A copy of this license is supposed to have been
 given to you along with TwinTeethMC so you can know your rights and
 responsibilities.  It should be in a file named COPYING.  Among other
 things, the copyright notice and this notice must be preserved on all
 copies.
 
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
import controlP5.*;

int JOG_X_POS = 35;
int JOG_Y_POS = 100;

int JOG_SCALE = 3;
int JOGX_MAX_POS = 33;
int JOGX_MIN_POS = -33;
int JOGY_MAX_POS = 40;
int JOGY_MIN_POS = -40;

int JOGX_MAX_LENGTH = (JOGX_MAX_POS - JOGX_MIN_POS);
int JOGY_MAX_LENGTH = (JOGY_MAX_POS - JOGY_MIN_POS);

int JOG_MAX_WIDTH = JOGX_MAX_LENGTH*JOG_SCALE;
int JOG_MAX_HEIGHT= JOGY_MAX_LENGTH*JOG_SCALE;

int WJOGX_MIN_POS = 0;
int WJOGX_MAX_POS = JOGX_MAX_LENGTH;

int WJOGY_MIN_POS = 0;
int WJOGY_MAX_POS = JOGY_MAX_LENGTH;

int WJOGX_CENTER_POS = WJOGX_MAX_POS/2;
int WJOGY_CENTER_POS = WJOGY_MAX_POS/2;

int X_JOG_GROUP = 501;
int Y_JOG_GROUP = 502;
int Z_JOG_GROUP = 503;

int X_JOG_CENTER_POSITION = JOG_X_POS + JOG_MAX_WIDTH/2;
int Y_JOG_CENTER_POSITION = JOG_Y_POS + (JOG_MAX_HEIGHT/2)-10;

int BUTTON_OFFSET = 20;
int Z_XOFFSET = 140;
int Y_JOG_TOP = Y_JOG_CENTER_POSITION - 110;
int X_TOOL_IMAGE = 500+92;
int Y_TOOL_IMAGE = Y_JOG_TOP;

float XY_SMALLMOVE_VALUE = 1;

Slider2D jogCtrl;
Button calibrateCtrl;
ScrollableList stepCtrl;
Button leftJogCtrl, rightJogCtrl, upJogCtrl, downJogCtrl;
Button zplus01, zplus1, zplus10, zplus100;
Button zmin01, zmin1, zmin10, zmin100;
Button homeCtrl, gotoZero, centerXY, zetaZero, setZeroCtrl, resetToolCtrl, bedLevelCtrl;
Slider xyFeedCtrl, zFeedCtrl;
Slider[] zMinCtrl = new Slider[MAX_TOOL_NUMBER];
Textlabel feedLabelCtrl;

String s = new String();

/**
 ** Jog control initialization
 **/
void JOGinit() {

  jogCtrl = cp5.addSlider2D("JogControl")
    .setCaptionLabel("")
      .setPosition(JOG_X_POS, JOG_Y_POS)
        .setSize(WJOGX_MAX_POS*JOG_SCALE, WJOGY_MAX_POS*JOG_SCALE)
          .setMinX(WJOGX_MIN_POS) 
            .setMaxX(WJOGX_MAX_POS) 
              .setMinY(WJOGY_MIN_POS)
                .setMaxY(WJOGY_MAX_POS);

  jogCtrl.moveTo("global");
  cp5.getProperties().move(jogCtrl, "default", "Nothing");



  // add a callback listener to adjust the value-label of the slider
  // for each value change.
  CallbackListener jogCtrlLabel = new CallbackListener() {
    public void controlEvent(CallbackEvent c) {
      if (c.getAction()==ControlP5.ACTION_RELEASE || c.getAction()==ControlP5.ACTION_BROADCAST) {
        if (c.getAction()==ControlP5.ACTION_RELEASE) {
          absMove((jogCtrl.getArrayValue()[0]-WJOGX_CENTER_POS), (jogCtrl.getArrayValue()[1]-WJOGY_CENTER_POS)*-1, (int)xyFeedCtrl.getValue());
       } 
        jogCtrl.getValueLabel().setText("x:"+ String.format("%2.2f", (jogCtrl.getArrayValue()[0]-WJOGX_CENTER_POS)) + " y:"+ String.format("%2.2f", (jogCtrl.getArrayValue()[1]-WJOGY_CENTER_POS)*-1));
        jogCtrl.getValueLabel().setFont(createFont("Arial", 10));
      }
    }
  };

  jogCtrl.addCallback(jogCtrlLabel);

  jogCtrl.setArrayValue(new float[] {
    WJOGX_CENTER_POS, WJOGY_CENTER_POS
  }
  );

  // The step list box

  stepCtrl = cp5.addScrollableList("Step")
    .setPosition(JOG_X_POS-30, JOG_Y_POS)
      .setSize(25, 20*5)
        .setItemHeight(20)
          .setBarHeight(20)
            .setColorBackground(BLUE)
              .setColorActive(RED)
                ;
  stepCtrl.getCaptionLabel().getStyle().marginTop = 4;
  stepCtrl.addItem("0.05", 0);
  stepCtrl.addItem("0.1", 1);
  stepCtrl.addItem("1", 2);
  stepCtrl.addItem("10", 3);
  stepCtrl.addItem("40", 4);
  stepCtrl.setValue(1);
  stepCtrl.moveTo("global");
  cp5.getProperties().move(stepCtrl, "default", "Nothing");

  // Center X/Y

  centerXY= cp5.addButton("CenterXY")
    .setLabel("Center X/Y")
      .setPosition(JOG_X_POS+(WJOGX_MAX_POS*JOG_SCALE)-65, Y_JOG_TOP-25)
        .setSize(65, 20)
          .setBroadcast(false)
            .setValue(0)
              .setColorBackground(BLUE)
                .setColorActive(RED)
                  .setBroadcast(true)
                    .updateSize();

  centerXY.moveTo("global");
  centerXY.getCaptionLabel().setFont(createFont("Arial Bold", 10));
  centerXY.getCaptionLabel().toUpperCase(false);

  // Goto zero

  gotoZero = cp5.addButton("GotoZero")
    .setLabel("Goto Zero")
      .setPosition(JOG_X_POS, Y_JOG_TOP-25)
        .setSize(65, 20)

          .setBroadcast(false)
            .setValue(0)
              .setColorBackground(BLUE)
                .setColorActive(RED)
                  .setBroadcast(true)
                    .updateSize();

  gotoZero.moveTo("global");
  gotoZero.getCaptionLabel().setFont(createFont("Arial Bold", 10));
  gotoZero.getCaptionLabel().toUpperCase(false);

  leftJogCtrl = cp5.addButton("LeftJogButton")
    .setImages(loadImage("Arrow-Left0.png"), loadImage("Arrow-Left1.png"), loadImage("Arrow-Left2.png") )
      .setPosition(JOG_X_POS-30, JOG_Y_POS+(JOG_MAX_HEIGHT/2)-11)
        .setBroadcast(false)
          .setId(X_JOG_GROUP)
            .setValue(-XY_SMALLMOVE_VALUE)
              .setBroadcast(true)
                .updateSize();

  leftJogCtrl.moveTo("global");

  rightJogCtrl = cp5.addButton("RightJogButton")
    .setImages(loadImage("Arrow-Right0.png"), loadImage("Arrow-Right1.png"), loadImage("Arrow-Right2.png") )
      .setPosition(JOG_X_POS+(JOG_MAX_WIDTH)+6, JOG_Y_POS+(JOG_MAX_HEIGHT/2)-11)
        .setBroadcast(false)
          .setId(X_JOG_GROUP)
            .setValue(+XY_SMALLMOVE_VALUE)
              .setBroadcast(true)
                .updateSize();

  rightJogCtrl.moveTo("global");

  upJogCtrl = cp5.addButton("UpJogButton")
    .setImages(loadImage("Arrow-Up0.png"), loadImage("Arrow-Up1.png"), loadImage("Arrow-Up2.png") )
      .setPosition(JOG_X_POS+(JOG_MAX_WIDTH/2)-11, JOG_Y_POS-29)
        .setBroadcast(false)
          .setId(Y_JOG_GROUP)
            .setValue(-XY_SMALLMOVE_VALUE)
              .setBroadcast(true)
                .updateSize();
  upJogCtrl.moveTo("global");   

  downJogCtrl = cp5.addButton("DownJogButton")
    .setImages(loadImage("Arrow-Down0.png"), loadImage("Arrow-Down1.png"), loadImage("Arrow-Down2.png") )
      .setPosition(JOG_X_POS+(JOG_MAX_WIDTH/2)-10, JOG_Y_POS+(JOG_MAX_HEIGHT)+6)
        .setBroadcast(false)
          .setId(Y_JOG_GROUP)
            .setValue(+XY_SMALLMOVE_VALUE)
              .setBroadcast(true)
                .updateSize();
  downJogCtrl.moveTo("global");       


  // Z min

  zmin01 = cp5.addButton("zmin01")
    .setPosition(X_JOG_CENTER_POSITION+Z_XOFFSET, Y_JOG_CENTER_POSITION+2+BUTTON_OFFSET)
      .setImages(loadImage("data/ZVerBar.png"), loadImage("data/A01.png"), loadImage("data/Refresh.png"))
        .setBroadcast(false)
          .setId(Z_JOG_GROUP)
            .setValue(+0.1)
              .setBroadcast(true)
                .updateSize();
  zmin01.moveTo("global");

  zmin1 = cp5.addButton("zmin1")
    .setPosition(X_JOG_CENTER_POSITION+Z_XOFFSET, Y_JOG_CENTER_POSITION+2+BUTTON_OFFSET*2)
      .setImages(loadImage("data/ZVerBar.png"), loadImage("data/A1.png"), loadImage("data/Refresh.png"))
        .setBroadcast(false)
          .setId(Z_JOG_GROUP)
            .setValue(+1)
              .setBroadcast(true)
                .updateSize();
  zmin1.moveTo("global");

  zmin10 = cp5.addButton("zmin10")
    .setPosition(X_JOG_CENTER_POSITION+Z_XOFFSET, Y_JOG_CENTER_POSITION+2+BUTTON_OFFSET*3)
      .setImages(loadImage("data/ZArrow-Down.png"), loadImage("data/A10.png"), loadImage("data/Refresh.png"))
        .setBroadcast(false)
          .setId(Z_JOG_GROUP)
            .setValue(+10)
              .setBroadcast(true)
                .updateSize();
  zmin10.moveTo("global");

  zmin100 = cp5.addButton("zmin100")
    .setPosition(X_JOG_CENTER_POSITION+Z_XOFFSET, Y_JOG_CENTER_POSITION+4+BUTTON_OFFSET*4)
      .setImage(loadImage("data/z+.png"))
        .setBroadcast(false)
          .lock()
            .updateSize();

  zmin100.moveTo("global");


  // Zplus

  zplus01 = cp5.addButton("zplus01")
    .setPosition(X_JOG_CENTER_POSITION+Z_XOFFSET, Y_JOG_CENTER_POSITION-2-BUTTON_OFFSET)
      .setImages(loadImage("data/ZVerBar.png"), loadImage("data/A01.png"), loadImage("data/Refresh.png"))
        .setBroadcast(false)
          .setId(Z_JOG_GROUP)
            .setValue(-0.1)
              .setBroadcast(true)
                .updateSize();
  zplus01.moveTo("global");

  zplus1 = cp5.addButton("zplus1")
    .setPosition(X_JOG_CENTER_POSITION+Z_XOFFSET, Y_JOG_CENTER_POSITION-2-BUTTON_OFFSET*2)
      .setImages(loadImage("data/ZVerBar.png"), loadImage("data/A1.png"), loadImage("data/Refresh.png"))
        .setBroadcast(false)
          .setId(Z_JOG_GROUP)
            .setValue(-1)
              .setBroadcast(true)
                .updateSize();
  zplus1.moveTo("global");

  zplus10 = cp5.addButton("zplus10")
    .setPosition(X_JOG_CENTER_POSITION+Z_XOFFSET, Y_JOG_CENTER_POSITION-2-BUTTON_OFFSET*3)
      .setImages(loadImage("data/ZArrow-Up.png"), loadImage("data/A10.png"), loadImage("data/Refresh.png"))
        .setBroadcast(false)
          .setId(Z_JOG_GROUP)
            .setValue(-10)
              .setBroadcast(true)
                .updateSize();
  zplus10.moveTo("global");

  zplus100 = cp5.addButton("zplus100")
    .setPosition(X_JOG_CENTER_POSITION+Z_XOFFSET, Y_JOG_CENTER_POSITION-4-BUTTON_OFFSET*4)
      .setImage(loadImage("data/z-.png"))
        .setBroadcast(false)
          .lock()
            .updateSize();
  zplus100.moveTo("global");


  // Zzero

    zetaZero = cp5.addButton("ZetaZero")
    .setPosition(X_JOG_CENTER_POSITION+Z_XOFFSET, Y_JOG_CENTER_POSITION)
      .setImages(loadImage("data/z.png"), loadImage("data/Azero.png"), loadImage("data/Refresh.png"))
        .setBroadcast(false)
          .setBroadcast(true)
            .updateSize();
  zetaZero.moveTo("global");

  // Home all axis

  homeCtrl = cp5.addButton("HomeAxis")
    .setPosition(JOG_X_POS, JOG_Y_POS+JOG_MAX_HEIGHT+3)
      .setImages(loadImage("data/home.png"), loadImage("data/home1.png"), loadImage("data/Refresh.png"))
        .updateSize();

  homeCtrl.moveTo("global");

  // Some other buttons & controls



  setZeroCtrl = cp5.addButton("SetToolZero")
    .setLabel("Set program zero")
      .setBroadcast(false)
        .setPosition(490, Y_JOG_TOP)
          .setSize(95, 30)
            .setValue(3)
              .setBroadcast(true)
                .setColorBackground(BLUE)
                  .setColorActive(RED)
                    .lock()
                      .hide()
                        ;
  setZeroCtrl.moveTo("global");
  setZeroCtrl.getCaptionLabel().setFont(createFont("Arial Bold", 10));
  setZeroCtrl.getCaptionLabel().toUpperCase(false);

  // Some other buttons & controls

  bedLevelCtrl = cp5.addButton("AutoBedLevel")
    .setLabel("Bed auto leveling")
      .setBroadcast(false)
        .setPosition(490, Y_JOG_TOP+35)
          .setSize(95, 30)
            .setValue(4)
              .setBroadcast(true)
                .setColorBackground(BLUE)
                  .setColorActive(RED)
                    .lock()
                      .hide()
                        ;

  bedLevelCtrl.moveTo("global");
  bedLevelCtrl.getCaptionLabel().setFont(createFont("Arial Bold", 10));
  bedLevelCtrl.getCaptionLabel().toUpperCase(false);

  resetToolCtrl = cp5.addButton("ResetTool")
    .setLabel("Reset tool (G94)")
      .setBroadcast(false)
        .setPosition(490, Y_JOG_TOP+35*2)
          .setSize(95, 30)
            .setValue(3)
              .setBroadcast(true)
                .setColorBackground(RED)
                  .setColorActive(RED)
                    .lock()
                      .hide()
                        ;

  resetToolCtrl.moveTo("global");
  resetToolCtrl.getCaptionLabel().setFont(createFont("Arial Bold", 10));
  resetToolCtrl.getCaptionLabel().toUpperCase(false);

  // Feed rates sliders
  feedLabelCtrl = cp5.addTextlabel("FeedRate")
    .setText("Seek Rate\n(mm/min)")
      .setPosition(356, Y_JOG_TOP-30)
        .setColorValue(WHITE)
          .setFont(createFont("Arial Bold", 10))
            ;


  feedLabelCtrl.get().toUpperCase(false);
  feedLabelCtrl.moveTo("global");
  cp5.getProperties().move(feedLabelCtrl, "default", "Nothing");
  // XY Feedrate

  xyFeedCtrl=cp5.addSlider("XYFeedRate")
    .setLabel("x/y")
      .setPosition(350, Y_JOG_TOP)
        .setSize(15, 100)
          .setRange(0, 2500)
 //           .setValue(1500)
            .lock()
              ;

  // add a callback listener to adjust the value-label of the slider
  // for each value change and control when it is released
  CallbackListener xyFeedRateCallBack = new CallbackListener() {
    public void controlEvent(CallbackEvent c) {
      if (c.getAction()==ControlP5.ACTION_BROADCAST) {
        xyFeedCtrl.getValueLabel().setText(String.format("%d", (int)xyFeedCtrl.getValue()));
      } else if (c.getAction()==ControlP5.ACTION_RELEASE || c.getAction()==ControlP5.ACTION_RELEASE_OUTSIDE) {
        setDefSeekRate((int)xyFeedCtrl.getValue());
      }
    }
  };

  xyFeedCtrl.addCallback(xyFeedRateCallBack);
  xyFeedCtrl.setValue(1500);


  // move label to top
  //xyFeedCtrl.getCaptionLabel().align(ControlP5.LEFT, ControlP5.TOP_OUTSIDE).setPaddingX(0);

  // move control to tool tab

  xyFeedCtrl.moveTo("global");
  xyFeedCtrl.getCaptionLabel().setFont(createFont("Arial Bold", 10));


  // Z Feedrate

  zFeedCtrl=cp5.addSlider("ZFeedRate")
    .setLabel("z")
      .setPosition(405, Y_JOG_TOP)
        .setSize(15, 100)
          .setRange(0, 2500)
            .setValue(1000)
              .lock()
                ;
  // move label to top

  //zFeedCtrl.getCaptionLabel().align(ControlP5.LEFT, ControlP5.TOP_OUTSIDE).setPaddingX(0);
  // move control to tool tab

  zFeedCtrl.moveTo("global");
  zFeedCtrl.getCaptionLabel().setFont(createFont("Arial Bold", 10));


  // add a callback listener to adjust the value-label of the slider
  // for each value change.
  CallbackListener zadjustLabel = new CallbackListener() {
    public void controlEvent(CallbackEvent c) {
      if (c.getAction()==ControlP5.ACTION_BROADCAST) {
        zFeedCtrl.getValueLabel().setText(String.format("%d", (int)zFeedCtrl.getValue()));
      }
    }
  };

  zFeedCtrl.addCallback(zadjustLabel);
  zFeedCtrl.setValue(1000);

  // Fields with diferent values for each tool

  for (int i = 0; i < MAX_TOOL_NUMBER; i++) {

    // add a new properties set 'setZMinField'
    cp5.getProperties().addSet(toolPropID+activeTool);

    // Zmin Field

    zMinCtrl[i]=cp5.addSlider("ZMinField"+i)
      .setLabel("Z min\n(mm)")
        .setPosition(450, Y_JOG_TOP)
          .setSize(15, 100)
            .setRange(0, -4)
              .setScrollSensitivity(.1)
                .lock()
                  .hide()
                    ;
    // move control to tool tab
    if (i == 0)
      zMinCtrl[i].moveTo("default");
    else 
      zMinCtrl[i].moveTo("tab"+i);
    zMinCtrl[i].getCaptionLabel().setFont(createFont("Arial Bold", 10));
    zMinCtrl[i].getCaptionLabel().toUpperCase(false);
    // move controller field from the default set to the tool set to stored
    cp5.getProperties().move(zMinCtrl[i], "default", toolPropID+i);
  }

  // add a callback listener to send the Zmin command

  class ZMinCallBack implements CallbackListener {
    public void controlEvent(CallbackEvent c) {
      if (c.getAction()==ControlP5.ACTION_RELEASE || c.getAction()==ControlP5.ACTION_RELEASE_OUTSIDE) {
        setZMin(c.getController().getValue());
      }
    }
  };


  for (int i = 0; i < MAX_TOOL_NUMBER; i++) {
    zMinCtrl[i].addCallback(new ZMinCallBack());
  }


  // Calibrate toogle activate the last buttons

  calibrateCtrl = cp5.addButton("Calibrate")
    .setPosition(X_TOOL_IMAGE, Y_TOOL_IMAGE-30)
      .setSize(48, 20)
        .setSwitch(true)
          //          .setOff()
          //           .setValue(0)
          .setColorBackground(BLUE)
            .setColorActive(GREEN)
              .lock()
                ;
  calibrateCtrl.moveTo("global");
  calibrateCtrl.getCaptionLabel().setFont(createFont("Arial Bold", 10));
  calibrateCtrl.getCaptionLabel().toUpperCase(false);


  // Lock the jog

  lockJog(true);
}


/**
 ** Calibrate() - Calibrate button
 **/
void Calibrate(boolean v) {
  if (calibrateCtrl.isOn()) {
    for (int i = 0; i < MAX_TOOL_NUMBER; i++) 
      zMinCtrl[i].show();
    setZeroCtrl.show();
    bedLevelCtrl.show();
    resetToolCtrl.show();
  } else {
    for (int i = 0; i < MAX_TOOL_NUMBER; i++) 
      zMinCtrl[i].hide();
    setZeroCtrl.hide();
    bedLevelCtrl.hide();
    resetToolCtrl.hide();
  }
}


/**
 ** lockJog() - lock/unlock jog
 **/
void lockJog(boolean status) {

  if (status) {

    jogCtrl.lock();
    stepCtrl.hide();
    leftJogCtrl.lock();
    rightJogCtrl.lock();
    upJogCtrl.lock();
    downJogCtrl.lock();
    zplus01.lock(); 
    zplus1.lock(); 
    zplus10.lock(); 
    zplus100.lock();
    zmin01.lock(); 
    zmin1.lock(); 
    zmin10.lock(); 
    zmin100.lock();
    gotoZero.lock(); 
    zetaZero.lock();
    centerXY.lock();
    homeCtrl.lock();
    xyFeedCtrl.lock();
    zFeedCtrl.lock();
    setZeroCtrl.lock();
    bedLevelCtrl.lock();
    resetToolCtrl.lock();
    for (int i = 0; i < MAX_TOOL_NUMBER; i++) {
      zMinCtrl[i].lock();
    }
    calibrateCtrl.setValue(0);
    calibrateCtrl.lock();
  } else {
    jogCtrl.unlock();
    stepCtrl.show();
    leftJogCtrl.unlock();
    rightJogCtrl.unlock();
    upJogCtrl.unlock();
    downJogCtrl.unlock();
    zplus01.unlock(); 
    zplus1.unlock(); 
    zplus10.unlock(); 
    zmin01.unlock(); 
    zmin1.unlock(); 
    zmin10.unlock(); 
    gotoZero.unlock(); 
    zetaZero.unlock();
    centerXY.unlock();
    homeCtrl.unlock();
    xyFeedCtrl.unlock();
    zFeedCtrl.unlock();
    setZeroCtrl.unlock();
    bedLevelCtrl.unlock();
    resetToolCtrl.unlock();
    for (int i = 0; i < MAX_TOOL_NUMBER; i++) {
      zMinCtrl[i].unlock();
    }
    calibrateCtrl.unlock();
  }
}
