/*
Copyright (c) 2015  V.Ayllón.  All rights reserved.
 
 TwinTeethMC is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
 to anyone for the consequences of using it or for whether it serves any
 particular purpose or works at all, unless he says so in writing.  Refer
 to the GNU General Public License, Version 2 or later, for full details.
 
 Everyone is granted permission to copy, modify and redistribute
 TwinTeethMC, but only under the conditions described in the GNU General Public
 License.  A copy of this license is supposed to have been
 given to you along with TwinTeethMC so you can know your rights and
 responsibilities.  It should be in a file named COPYING.  Among other
 things, the copyright notice and this notice must be preserved on all
 copies.
 
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
import org.ghost4j.*;

import java.awt.Image;
import java.awt.image.RenderedImage;
import java.awt.image.BufferedImage;
import java.awt.Graphics;
import java.io.File;
import java.io.IOException;
import java.util.List;

import javax.imageio.*;

import org.ghost4j.document.PSDocument;
import org.ghost4j.renderer.SimpleRenderer;

import org.ghost4j.Ghostscript;
import org.ghost4j.GhostscriptException;
import org.ghost4j.display.ImageWriterDisplayCallback;

String PSTEMPFILENAME = tempDirectory+ "/pstempfile.bmp";
String imageFilename;


/**
 ** PostScriptToBMP() - Convert a postscrip file to BMP
 ** fileWidth and fileHeight in mm
 **/
public void PostScriptToBMP(String fileName, float fileWidth, float fileHeight) throws GhostscriptException, IOException {
  float dpmm = 72/25.4;  // Dots per mm in PS document
  //get Ghostscript instance
  Ghostscript gs = Ghostscript.getInstance();

  //create display callback (capture display output pages as images)
  ImageWriterDisplayCallback displayCallback = new ImageWriterDisplayCallback();

  //set display callback
  gs.setDisplayCallback(displayCallback);

  //prepare Ghostscript interpreter parameters with display device
  String[] gsArgs = new String[10];
  gsArgs[0] = "-dQUIET";
  gsArgs[1] = "-dNOPAUSE";
  gsArgs[2] = "-dBATCH";
  gsArgs[3] = "-dSAFER";
  gsArgs[4] = "-sDEVICE=display";
  gsArgs[5] = "-sDisplayHandle=0";
  gsArgs[6] = "-dDisplayFormat=16#804";
  gsArgs[7] = "-dDEVICEWIDTHPOINTS="+fileWidth*dpmm; 
  gsArgs[8] = "-dDEVICEHEIGHTPOINTS="+fileHeight*dpmm; 
  gsArgs[9] = "-r"+raster.DPI; 

  //run PostScript 
  synchronized (gs) {
    gs.initialize(gsArgs);
    gs.runFile(fileName);
    gs.exit();
  }
  if (displayCallback.getImages().size() > 0) {
    ImageIO.write((RenderedImage) displayCallback.getImages().get(0), "bmp", new File(PSTEMPFILENAME));
  }
}
